/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/


#include "mxsmbus.h"

//#define SMBUS_SLAVE_ADDRESS 0x58 //0xB0
#define AVALON_SLAVE_SELECT_SWITCH 0x1
#define AVALON_SLAVE_SELECT_PORT0 0x2
#define AVALON_SLAVE_SELECT_PORT1 0x4
#define AVALON_SLAVE_SELECT_PORT2 0x8

#define PORT_CFG_REG_GROUP_OFFSET_GEN 0x0000
#define PORT_CFG_REG_GROUP_OFFSET_HSR 0x1000
#define PORT_CFG_REG_GROUP_OFFSET_PTP 0x2000
#define PORT_CFG_REG_GROUP_OFFSET_CNT 0x3000
#define PORT_CFG_REG_GROUP_OFFSET_IPO 0x4000

#define GEN_CFG_REG_OFFSET_PORT_STATE 0x0000 //PORT_STATE register in general configuration and state register
#define GEN_CFG_REG_OFFSET_PORT_VLAN 0x0008 //PORT_VLAN register in general configuration and state register
#define GEN_CFG_REG_OFFSET_VLAN0_MAPPING 0x0009 //VLAN0_MAPPING register in general configuration and state register
#define GEN_CFG_REG_OFFSET_PORT_FWD_MASK 0x000A //PORT_FWD_MASK register in general configuration and state register
#define GEN_CFG_REG_OFFSET_PORT_VLAN_PRIO 0x000B //PORT_VLAN_PRIO register in general configuration and state register

#define HSR_CFG_REG_OFFSET_HSR_CFG 0x0000 //HSR_CFG register in HSR configuration and state register

#define CNT_CFG_REG_OFFSET_CNT_CTRL 0x0000 //CNT_CTRL register in CNT configuration and state register
#define CNT_CFG_REG_OFFSET_RX_GOOD_OCTETS_L 0x0100 //
#define CNT_CFG_REG_OFFSET_RX_GOOD_OCTETS_H 0x0101 //
#define CNT_CFG_REG_OFFSET_RX_BAD_OCTETS_L 0x0102 //
#define CNT_CFG_REG_OFFSET_RX_BAD_OCTETS_H 0x0103 //
#define CNT_CFG_REG_OFFSET_RX_UNICAST_L 0x0104 //
#define CNT_CFG_REG_OFFSET_RX_UNICAST_H 0x0105 //
#define CNT_CFG_REG_OFFSET_RX_BROADCAST_L 0x0106 //
#define CNT_CFG_REG_OFFSET_RX_BROADCAST_H 0x0107 //
#define CNT_CFG_REG_OFFSET_RX_MULTICAST_L 0x0108 //
#define CNT_CFG_REG_OFFSET_RX_MULTICAST_H 0x0109 //
#define CNT_CFG_REG_OFFSET_RX_UNDERSIZE_L 0x010A //
#define CNT_CFG_REG_OFFSET_RX_UNDERSIZE_H 0x010B //
#define CNT_CFG_REG_OFFSET_RX_FRAGMENTS_L 0x010C //
#define CNT_CFG_REG_OFFSET_RX_FRAGMENTS_H 0x010D //
#define CNT_CFG_REG_OFFSET_RX_OVERSIZE_L 0x010E //
#define CNT_CFG_REG_OFFSET_RX_OVERSIZE_H 0x010F //
#define CNT_CFG_REG_OFFSET_RX_JABBER_L 0x0110 //
#define CNT_CFG_REG_OFFSET_RX_JABBER_H 0x0111 //
#define CNT_CFG_REG_OFFSET_RX_ERR_L 0x0112 //
#define CNT_CFG_REG_OFFSET_RX_ERR_H 0x0113 //
#define CNT_CFG_REG_OFFSET_RX_CRC_L 0x0114 //
#define CNT_CFG_REG_OFFSET_RX_CRC_H 0x0115 //
#define CNT_CFG_REG_OFFSET_RX_64_L 0x0116 //
#define CNT_CFG_REG_OFFSET_RX_64_H 0x0117 //
#define CNT_CFG_REG_OFFSET_RX_65_127_L 0x0118 //
#define CNT_CFG_REG_OFFSET_RX_65_127_H 0x0119 //
#define CNT_CFG_REG_OFFSET_RX_128_255_L 0x011A //
#define CNT_CFG_REG_OFFSET_RX_128_255_H 0x011B //
#define CNT_CFG_REG_OFFSET_RX_256_511_L 0x011C //
#define CNT_CFG_REG_OFFSET_RX_256_511_H 0x011D //
#define CNT_CFG_REG_OFFSET_RX_512_1023_L 0x011E //
#define CNT_CFG_REG_OFFSET_RX_512_1023_H 0x011F //
#define CNT_CFG_REG_OFFSET_RX_1024_1536_L 0x0120 //
#define CNT_CFG_REG_OFFSET_RX_1024_1536_H 0x0121 //
#define CNT_CFG_REG_OFFSET_RX_HSRPRP_L 0x0122 //
#define CNT_CFG_REG_OFFSET_RX_HSRPRP_H 0x0123 //
#define CNT_CFG_REG_OFFSET_RX_WRONGLAN_L 0x0124 //
#define CNT_CFG_REG_OFFSET_RX_WRONGLAN_H 0x0125 //
#define CNT_CFG_REG_OFFSET_RX_DUPLICATE_L 0x0126 //
#define CNT_CFG_REG_OFFSET_RX_DUPLICATE_H 0x0127 //

#define CNT_CFG_REG_OFFSET_TX_OCTETS_L 0x0140 //
#define CNT_CFG_REG_OFFSET_TX_OCTETS_H 0x0141 //
#define CNT_CFG_REG_OFFSET_TX_UNICAST_L 0x0142 //
#define CNT_CFG_REG_OFFSET_TX_UNICAST_H 0x0143 //
#define CNT_CFG_REG_OFFSET_TX_BROADCAST_L 0x0144 //
#define CNT_CFG_REG_OFFSET_TX_BROADCAST_H 0x0145 //
#define CNT_CFG_REG_OFFSET_TX_MULTICAST_L 0x0146 //
#define CNT_CFG_REG_OFFSET_TX_MULTICAST_H 0x0147 //
#define CNT_CFG_REG_OFFSET_TX_HSRPRP_L 0x0148 //
#define CNT_CFG_REG_OFFSET_TX_HSRPRP_H 0x0149 //
#define CNT_CFG_REG_OFFSET_PRIQ_DROP_L 0x0160 //
#define CNT_CFG_REG_OFFSET_PRIQ_DROP_H 0x0061 //
#define CNT_CFG_REG_OFFSET_EARLY_DROP_L 0x0062 //
#define CNT_CFG_REG_OFFSET_EARLY_DROP_H 0x0063 //

#define CMD_CODE_READ_AVALON_DATA 0x50 //0x010XAAAA
#define CMD_CODE_WRITE_AVALON_DATA 0x30 //0x001XAAAA
#define DATACOUNT_WRITE 4
#define DATACOUNT_READ 2

/*******************************************************
*
* Structure of Generic Configuration and State Registers
*
********************************************************/
typedef struct _BIT_GEN_CFG_PORT_STATE
{
	unsigned short speed_select	 : 2; //Bit 8
	unsigned short current_speed : 2;
	unsigned short reserved2	 : 4; //Bit 15

	unsigned short port_frwd_st	 : 2; //Bit 0
	unsigned short port_mgmt_st	 : 2;
	unsigned short port_hw_mode	 : 2;
	unsigned short reserved		 : 2; //Bit7
}
BIT_GEN_CFG_PORT_STATE, *PBIT_GEN_CFG_PORT_STATE;

typedef union _GEN_CFG_PORT_STATE
{
	unsigned short regData;
	BIT_GEN_CFG_PORT_STATE bitInfo;

}GEN_CFG_PORT_STATE, *PGEN_CFG_PORT_STATE;

/*******************************************************
*
* Structure of HSR Registers
*
********************************************************/
typedef struct _BIT_HSR_REG
{
	unsigned short hsr_prp_mode		: 1; //Bit 8
	unsigned short red_intlnk_mode	: 1;
	unsigned short lan_id			: 1;
	unsigned short net_id			: 3; //PathId = NetId + LanId
	unsigned short reserved2		: 2; //Bit 15

	unsigned short port_mode		: 1; //Bit 7
	unsigned short reserved			: 7; //Bit 0 
}
BIT_BIT_HSR_REG, *PBIT_HSR_REG;

typedef union _HSR_REG
{
	unsigned short regData;
	BIT_BIT_HSR_REG bitInfo;

}HSR_REG, *PHSR_REG;



#ifdef __cplusplus    // If used by C++ code, 
extern "C" {          // we need to export the C interface
#endif

/*
*
* Functions to set/get Avalon register
*
*/
int SetAvalonReg(UINT8 slaveAddr, UINT8 portSel, UINT8 *avalonAddr, UINT8 *avalonData);
int GetAvalonReg(UINT8 slaveAddr, UINT8 portSel, UINT8 *avalonAddr, UINT8 *avalonData);

/*
*
* Low level functions to set/get HSR register
*
*/
int WriteAvalonData(UINT8 slaveAddr, UINT8 portSel, UINT8 *smbusBlockData);
int ReadAvalonData(UINT8 slaveAddr, UINT8 portSel, UINT8 *smbusProcData);


#ifdef __cplusplus    // If used by C++ code, 
}
#endif